package com.qucheng.game.data.oldsystem.ods;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.qucheng.game.data.oldsystem.serialization.CustomerDeserializationSchema;
import com.qucheng.game.data.oldsystem.sink.ads_sink;
import com.ververica.cdc.connectors.mysql.source.MySqlSource;
import com.ververica.cdc.connectors.mysql.table.StartupOptions;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.MapFunction;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;

import java.util.Arrays;
import java.util.List;

/**
 * ODS层数据填充
 * 过滤字段及转换字段
 */
public class OdsDataFilling {

    public static void main(String[] args) throws Exception {
        System.setProperty("HADOOP_USER_NAME", "flink");
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        env.setParallelism(1);


        //2.通过FlinkCDC构建SourceFunction并读取数据
        MySqlSource<String> mySqlSource = MySqlSource.<String>builder()
                .hostname("118.31.103.66")
                .port(3306)
                .username("root")
                .password("Qc_1234567")
                .databaseList("ods")
                .tableList("ods.byte_t_ad_data_day")
                .deserializer(new CustomerDeserializationSchema())
                //5400 和 6400
                .startupOptions(StartupOptions.initial())
                .build();

        DataStreamSource<String> dataStreamSource = env.fromSource(mySqlSource, WatermarkStrategy.noWatermarks(), "ODS_Mysql_Source");

        dataStreamSource.map(new FieldFilterMap()).addSink(new ads_sink());


        //4.启动任务
        env.execute("GAME_ODS_FILLING");
    }

    public static class FieldFilterMap implements MapFunction<String, JSONObject> {
        @Override
        public JSONObject map(String value) throws Exception {

            JSONObject result = new JSONObject();
            JSONObject data = JSON.parseObject(value);

            String tableName = data.getString("tableName");
            JSONObject after = data.getJSONObject("after");
            JSONObject key = data.getJSONObject("key");
            String afterFinal = "";
            String keyFinal = "";
            String tableNameFinal = "";

            if (tableName.equals("byte_t_ad_data_day")) {

                List<String> columns = Arrays.asList("account_id", "day",
                        "cost", "show", "click", "ctr");

                after.entrySet().removeIf(next -> !columns.contains(next.getKey()));

                key.remove("ad_id");

                keyFinal = key.toString().replaceAll("\"day\":", "\"date\":");

                afterFinal = after.toString().replaceAll("\"day\":", "\"date\":")
                        .replaceAll("\"show\":", "\"view_count\":")
                        .replaceAll("\"click\":", "\"valid_click_count\":");


                tableNameFinal = "daily_tt";
            }

            Object afterFinalJson = JSONObject.parse(afterFinal);
            Object keyFinalJson = JSONObject.parse(keyFinal);

            result.put("after", afterFinalJson);
            result.put("key", keyFinalJson);
            result.put("type", data.getString("type"));
            result.put("tableName", tableNameFinal);
            result.put("db", "quchen_text");

            return result;
        }
    }
}